@@ -22,6 +22,7 @@ create table @NAMESPACE@.sl_node (
 	no_id				int4,
 	no_active			bool,
 	no_comment			text,
+	no_failed			bool,
 	CONSTRAINT "sl_node-pkey"
 		PRIMARY KEY (no_id)
 ) WITHOUT OIDS;
@@ -369,8 +370,11 @@ create table @NAMESPACE@.sl_log_1 (
 	log_txid			bigint,
 	log_tableid			int4,
 	log_actionseq		int8,
-	log_cmdtype			char,
-	log_cmddata			text
+	log_tablenspname	text,
+	log_tablerelname	text,
+	log_cmdtype			"char",
+	log_cmdupdncols		int4,
+	log_cmdargs			text[]
 ) WITHOUT OIDS;
 create index sl_log_1_idx1 on @NAMESPACE@.sl_log_1
 	(log_origin, log_txid, log_actionseq);
@@ -383,8 +387,12 @@ comment on table @NAMESPACE@.sl_log_1 is 'Stores each change to be propagated to
 comment on column @NAMESPACE@.sl_log_1.log_origin is 'Origin node from which the change came';
 comment on column @NAMESPACE@.sl_log_1.log_txid is 'Transaction ID on the origin node';
 comment on column @NAMESPACE@.sl_log_1.log_tableid is 'The table ID (from sl_table.tab_id) that this log entry is to affect';
-comment on column @NAMESPACE@.sl_log_1.log_cmdtype is 'Replication action to take. U = Update, I = Insert, D = DELETE';
-comment on column @NAMESPACE@.sl_log_1.log_cmddata is 'The data needed to perform the log action';
+comment on column @NAMESPACE@.sl_log_1.log_actionseq is 'The sequence number in which actions will be applied on replicas';
+comment on column @NAMESPACE@.sl_log_1.log_tablenspname is 'The schema name of the table affected';
+comment on column @NAMESPACE@.sl_log_1.log_tablerelname is 'The table name of the table affected';
+comment on column @NAMESPACE@.sl_log_1.log_cmdtype is 'Replication action to take. U = Update, I = Insert, D = DELETE, T = TRUNCATE';
+comment on column @NAMESPACE@.sl_log_1.log_cmdupdncols is 'For cmdtype=U the number of updated columns in cmdargs';
+comment on column @NAMESPACE@.sl_log_1.log_cmdargs is 'The data needed to perform the log action on the replica';
 
 -- ----------------------------------------------------------------------
 -- TABLE sl_log_2
@@ -394,15 +402,12 @@ create table @NAMESPACE@.sl_log_2 (
 	log_txid			bigint,
 	log_tableid			int4,
 	log_actionseq		int8,
-	log_cmdtype			char,
-	log_cmddata			text
+	log_tablenspname	text,
+	log_tablerelname	text,
+	log_cmdtype			"char",
+	log_cmdupdncols		int4,
+	log_cmdargs			text[]
 ) WITHOUT OIDS;
-comment on table @NAMESPACE@.sl_log_2 is 'Stores each change to be propagated to subscriber nodes';
-comment on column @NAMESPACE@.sl_log_2.log_origin is 'Origin node from which the change came';
-comment on column @NAMESPACE@.sl_log_2.log_txid is 'Transaction ID on the origin node';
-comment on column @NAMESPACE@.sl_log_2.log_tableid is 'The table ID (from sl_table.tab_id) that this log entry is to affect';
-comment on column @NAMESPACE@.sl_log_2.log_cmdtype is 'Replication action to take. U = Update, I = Insert, D = DELETE';
-comment on column @NAMESPACE@.sl_log_2.log_cmddata is 'The data needed to perform the log action';
 create index sl_log_2_idx1 on @NAMESPACE@.sl_log_2
 	(log_origin, log_txid, log_actionseq);
 
@@ -410,6 +415,36 @@ create index sl_log_2_idx1 on @NAMESPACE@.sl_log_2
 -- create index sl_log_2_idx2 on @NAMESPACE@.sl_log_2
 -- 	(log_txid);
 
+comment on table @NAMESPACE@.sl_log_2 is 'Stores each change to be propagated to subscriber nodes';
+comment on column @NAMESPACE@.sl_log_2.log_origin is 'Origin node from which the change came';
+comment on column @NAMESPACE@.sl_log_2.log_txid is 'Transaction ID on the origin node';
+comment on column @NAMESPACE@.sl_log_2.log_tableid is 'The table ID (from sl_table.tab_id) that this log entry is to affect';
+comment on column @NAMESPACE@.sl_log_2.log_actionseq is 'The sequence number in which actions will be applied on replicas';
+comment on column @NAMESPACE@.sl_log_2.log_tablenspname is 'The schema name of the table affected';
+comment on column @NAMESPACE@.sl_log_2.log_tablerelname is 'The table name of the table affected';
+comment on column @NAMESPACE@.sl_log_2.log_cmdtype is 'Replication action to take. U = Update, I = Insert, D = DELETE, T = TRUNCATE';
+comment on column @NAMESPACE@.sl_log_2.log_cmdupdncols is 'For cmdtype=U the number of updated columns in cmdargs';
+comment on column @NAMESPACE@.sl_log_2.log_cmdargs is 'The data needed to perform the log action on the replica';
+
+-- ----------------------------------------------------------------------
+-- TABLE sl_log_script
+-- ----------------------------------------------------------------------
+create table @NAMESPACE@.sl_log_script (
+	log_origin			int4,
+	log_txid			bigint,
+	log_actionseq		int8,
+	log_cmdtype			"char",
+	log_cmdargs			text[]
+) WITHOUT OIDS;
+create index sl_log_script_idx1 on @NAMESPACE@.sl_log_script
+	(log_origin, log_txid, log_actionseq);
+
+comment on table @NAMESPACE@.sl_log_script is 'Captures SQL script queries to be propagated to subscriber nodes';
+comment on column @NAMESPACE@.sl_log_script.log_origin is 'Origin name from which the change came';
+comment on column @NAMESPACE@.sl_log_script.log_txid is 'Transaction ID on the origin node';
+comment on column @NAMESPACE@.sl_log_script.log_actionseq is 'The sequence number in which actions will be applied on replicas';
+comment on column @NAMESPACE@.sl_log_2.log_cmdtype is 'Replication action to take. S = Script statement, s = Script complete';
+comment on column @NAMESPACE@.sl_log_script.log_cmdargs is 'The DDL statement, optionally followed by selected nodes to execute it on.';
 
 -- ----------------------------------------------------------------------
 -- TABLE sl_registry
@@ -427,6 +462,44 @@ comment on column @NAMESPACE@.sl_registry.reg_text is 'Option value if type text
 comment on column @NAMESPACE@.sl_registry.reg_timestamp is 'Option value if type timestamp';
 
 
+-- ----------------------------------------------------------------------
+-- TABLE sl_apply_stats
+-- ----------------------------------------------------------------------
+create table @NAMESPACE@.sl_apply_stats (
+	as_origin			int4,
+	as_num_insert		int8,
+	as_num_update		int8,
+	as_num_delete		int8,
+	as_num_truncate		int8,
+	as_num_script		int8,
+	as_num_total		int8,
+	as_duration			interval,
+	as_apply_first		timestamptz,
+	as_apply_last		timestamptz,
+	as_cache_prepare	int8,
+	as_cache_hit		int8,
+	as_cache_evict		int8,
+	as_cache_prepare_max int8
+) WITHOUT OIDS;
+
+create index sl_apply_stats_idx1 on @NAMESPACE@.sl_apply_stats
+	(as_origin);
+
+comment on table @NAMESPACE@.sl_apply_stats is 'Local SYNC apply statistics (running totals)';
+comment on column @NAMESPACE@.sl_apply_stats.as_origin is 'Origin of the SYNCs';
+comment on column @NAMESPACE@.sl_apply_stats.as_num_insert is 'Number of INSERT operations performed';
+comment on column @NAMESPACE@.sl_apply_stats.as_num_update is 'Number of UPDATE operations performed';
+comment on column @NAMESPACE@.sl_apply_stats.as_num_delete is 'Number of DELETE operations performed';
+comment on column @NAMESPACE@.sl_apply_stats.as_num_truncate is 'Number of TRUNCATE operations performed';
+comment on column @NAMESPACE@.sl_apply_stats.as_num_script is 'Number of DDL operations performed';
+comment on column @NAMESPACE@.sl_apply_stats.as_num_total is 'Total number of operations';
+comment on column @NAMESPACE@.sl_apply_stats.as_duration is 'Processing time';
+comment on column @NAMESPACE@.sl_apply_stats.as_apply_first is 'Timestamp of first recorded SYNC';
+comment on column @NAMESPACE@.sl_apply_stats.as_apply_last is 'Timestamp of most recent recorded SYNC';
+comment on column @NAMESPACE@.sl_apply_stats.as_cache_evict is 'Number of apply query cache evict operations';
+comment on column @NAMESPACE@.sl_apply_stats.as_cache_prepare_max is 'Maximum number of apply queries prepared in one SYNC group';
+
+
 -- **********************************************************************
 -- * Views
 -- **********************************************************************
@@ -444,9 +517,67 @@ create view @NAMESPACE@.sl_seqlastvalue as
 		where S.set_id = SQ.seq_set
 			and PGC.oid = SQ.seq_reloid and PGN.oid = PGC.relnamespace;
 		
+
+create view @NAMESPACE@.sl_failover_targets as
+select  set_id,
+	set_origin as set_origin,
+	sub1.sub_receiver as backup_id
+
+FROM
+	@NAMESPACE@.sl_subscribe sub1
+	,@NAMESPACE@.sl_set set1
+	where
+ 	sub1.sub_set=set_id
+	and sub1.sub_forward=true
+	--exclude candidates where the set_origin
+	--has a path a node but the failover
+	--candidate has no path to that node
+	and sub1.sub_receiver not in
+	    (select p1.pa_client from
+	    @NAMESPACE@.sl_path p1 
+	    left outer join @NAMESPACE@.sl_path p2 on
+	    (p2.pa_client=p1.pa_client 
+	    and p2.pa_server=sub1.sub_receiver)
+	    where p2.pa_client is null
+	    and p1.pa_server=set_origin
+	    and p1.pa_client<>sub1.sub_receiver
+	    )
+	and sub1.sub_provider=set_origin
+	--exclude any subscribers that are not
+	--direct subscribers of all sets on the
+	--origin
+	and sub1.sub_receiver not in
+	(select direct_recv.sub_receiver
+		from
+			
+		(--all direct receivers of the first set
+		select subs2.sub_receiver
+		from @NAMESPACE@.sl_subscribe subs2
+		where subs2.sub_provider=set1.set_origin
+		      and subs2.sub_set=set1.set_id) as
+		      direct_recv
+		inner join
+		(--all other sets from the origin
+		select set_id from @NAMESPACE@.sl_set set2
+		where set2.set_origin=set1.set_origin
+		and set2.set_id<>sub1.sub_set)
+		as othersets on(true)
+		left outer join @NAMESPACE@.sl_subscribe subs3
+		on(subs3.sub_set=othersets.set_id
+		   and subs3.sub_forward=true
+		   and subs3.sub_provider=set1.set_origin
+		   and direct_recv.sub_receiver=subs3.sub_receiver)
+	    where subs3.sub_receiver is null
+	    );
+
+		      
+	
+
+	    
+
 -- **********************************************************************
 -- * Sequences
--- **********************************************************************
+-- *********************************************************************
 
 
 -- ----------------------------------------------------------------------
@@ -592,9 +723,41 @@ comment on column @NAMESPACE@.sl_components.co_eventtype is 'what kind of event
 comment on column @NAMESPACE@.sl_components.co_event is 'which event have I started processing?';
 
 
+
+--
+-- we create a function + aggregate for string_agg to aggregate strings
+-- some versions of PG (ie prior to 9.0) don't support this
+CREATE OR replace function @NAMESPACE@.agg_text_sum(txt_before TEXT, txt_new TEXT) RETURNS TEXT AS
+$BODY$
+DECLARE
+  c_delim text;
+BEGIN
+    c_delim = ',';
+	IF (txt_before IS NULL or txt_before='') THEN
+	   RETURN txt_new;
+	END IF;
+	RETURN txt_before || c_delim || txt_new;
+END;
+$BODY$
+LANGUAGE plpgsql;
+comment on function @NAMESPACE@.agg_text_sum(text,text) is 
+'An accumulator function used by the slony string_agg function to
+aggregate rows into a string';
+--
+-- create a string_agg function in the slony schema.
+-- PG 8.3 does not have this function so we make our own
+-- when slony stops supporting PG 8.3 we can switch to
+-- the PG 9.0+ provided version of string_agg
+--
+CREATE AGGREGATE @NAMESPACE@.string_agg(text) (
+SFUNC=@NAMESPACE@.agg_text_sum,
+STYPE=text,
+INITCOND=''
+);
+
+
 -- ----------------------------------------------------------------------
 -- Last but not least grant USAGE to the replication schema objects.
 -- ----------------------------------------------------------------------
 grant usage on schema @NAMESPACE@ to public;
 
-
