@@ -16,14 +16,14 @@ CREATE TABLE table3(
 					ON UPDATE SET NULL ON DELETE SET NULL,
   mod_date	TIMESTAMPTZ	NOT NULL DEFAULT now(),
   data		FLOAT		NOT NULL DEFAULT random()
-  CONSTRAINT table3_date_check	CHECK (mod_date <= now())
+  CONSTRAINT table3_date_check	CHECK (mod_date <= now()),
+  primary key (id)
 ); 
 
 -- Table to perform UTF8 tests (checks multibyte; should be helpful in
 -- preventing problems with Asian character sets too)
-CREATE SEQUENCE utf8_id;
 CREATE TABLE utf8table (
-    id integer not null unique default nextval('utf8_id'),
+    id serial,
     string text,
     primary key(id)
 );
